-------------------------------
-- Classical SuperHero Index --
-------------------------------

function sh.expand(t)
	local function Format( ...)
		local str, m, n = [[]]
		for m,n in ipairs(arg) do str = str..tostring(n) end
		return str
	end
	return Format(Macro(t))
end

function sh.parse(cmd, ...)
	local p, id, val = [[]]
	for id, val in ipairs(arg) do p = p..[[ ]]..val end
	parse(parses[cmd]..p)
end

function sh.print(c, t)
	print(color[c]..[[[SH] ]]..t)
end

function sh.msg(c, t)
	local tl = [[]]
	if not string.find(t, [[@C]]) then tl = [[[SH] ]] end
	msg(color[c]..tl..sh.expand(t))
end

function sh.msg2(p, c, t)
	local tl = [[]]
	if not string.find(t, [[@C]]) then tl = [[[SH] ]] end
	msg2(p,color[c]..tl..sh.expand(t))
end

function sh.sv_msg2(p, t)
	msg2(p,sh.expand(t))
end

function sh.load.error(f, e)
	if (type(f) ~= [[string]]) then f = sh.file[f] end
	sh.print(2,[[File Error: "]]..sh_dir..f..[[" (]]..sh.error[e]..[[)]])
end

function sh.load.sucess(f, s)
	if (type(f) ~= [[string]]) then f = sh.file[f] end
	sh.print(1,[[File Sucess: "]]..sh_dir..f..[[" ]]..sh.sucess[s])
end

function sh.add.error(m, e)
	if type(m) == [[number]] then m = cfg.mark[m] end
	sh.print(2,[[SuperHeroes Config Error: "]]..m..[[" (]]..cfg.error[e]..[[)]])
end

function sh.set.error(m, e)
	if type(m) == [[number]] then m = cfg.h_func[m][1] end
	sh.print(2,[[SuperHeroes Config Error: "]]..m..[[" (]]..cfg.error[e]..[[)]])
end

function sh.member.error(e, ln)
	sh.print(2,[[Members Config Error: "#addmember" (]]..cfg.error[e]..[[ on ]]..ln..[[ line)]])
end

function sh.set.sucess(m, e)
	if (sh_config_success > 0) then 
		local p = [[]] ; table.remove(e, 1)
		for id, val in ipairs(e) do p = p..val..[[ ]] end
		sh.print(1,cfg.h_func[m]..[[: ]]..p)
	end
end

function split(t, b)
	if t then
		local cmd, match = {}, [[[^]]..b..[[]+]]
		for word in string.gmatch(t, match) do table.insert(cmd, word) end
		return cmd
	else
		return {nil}
	end
end

function sh.hudtxt2(p, s, c, t, x, y, a)
	sh.parse(2, p, s, [["]]..color[c]..sh.expand(t)..[["]], x, y, a)
end

function sh.hudtxt(s, c, t, x, y, a)
	sh.parse(1, s, [["]]..color[c]..sh.expand(t)..[["]], x, y, a)
end

function sh.distance(p, k, rad)
	local dist = false
	if ((((player(p,[[x]]) - player(k,[[x]])) ^ 2 + (player(p,[[y]]) - player(k,[[y]])) ^ 2) ^ .5) <= rad) then dist = true end
	return dist
end

function sh.supdate(t, ln)
     readcount[t] = readcount[t] + ln
     if readcount[t] > #t then
          readcount[t] = 1
     end
end

function sh.readbyte(t)
	local stream = s_get(t)
	local byte = getByte(stream)[1]
	sh.supdate(t, 1)
	return byte
end

function cos(a) return math.cos(math.rad(a)) end

function sin(a) return math.sin(math.rad(a)) end

function sh.load_image(path, x, y, m, p, b, a)
	local id = image([[gfx/classic_superhero/]]..path,x,y,m,p)
	imageblend(id,b) ; imagealpha(id,a)
	return id
end

function sh.oval_image(p, t, size, rot, p_mode, clr)
	if (rot > 0) then rot = 1 else rot = 0 end
	if (p_mode > 0) then p_mode = p else p_mode = 0 end
	local id = sh.load_image([[sh_oval_]]..t..[[.png]],rot,0,100 + p,p_mode,sh_oval_blend,sh_oval_alpha / 100)
	table.insert(pl.img_oval[p], id)
	imagescale(id,size / 112, size / 112)
	for m, n in pairs(sh.RGB) do
		if string.find(clr,m) then imagecolor(id, sh.RGB[m][1], sh.RGB[m][2], sh.RGB[m][3]) end
	end
end

function sh.throw_image(p, path, x , y, rot, mode)
	local id = sh.load_image(table.concat(split(path,[[ ]])),x,y,mode,0,0,1)
	imagepos(id, x, y, rot)
	table.insert(obj.img, id)
end

function sh.throw_collide(x, y)
	local colide = fase
	x, y = math.ceil(x / 32) - 1, math.ceil(y / 32) - 1
	if tile( x, y, [[wall]] ) then colide = true end
	return colide
end

function sh.obj_dist(x, y, k, rad)
	local dist = false
	if ((((x - player(k,[[x]])) ^ 2 + (y - player(k,[[y]])) ^ 2) ^ .5) <= rad) then dist = true end
	return dist
end

function sh.snd(s)
	if (sh_sound > 0) then parse([[sv_sound ]]..snd_dir..s..[[.ogg]]) end
end

function sh.snd2(p,s)
	if (sh_sound > 0) then parse([[sv_sound2 ]]..p..[[  ]]..snd_dir..s..[[.ogg]]) end
end

function sh.snd_quake(s)
	if (sh_sound > 0) then parse([[sv_sound ]]..ut_dir..s..[[.wav]]) end
end

function sh.setbuilding(type,x, y, p)
	if not tile(x,y,[[wall]]) and not entity(x,y,[[exists]]) then
		sh.parse(14,type, x, y, 0, 1, player(p,[[team]]), p)
	end
end

local _mtg = getmetatable(_G) or {}
function _mtg.__index(self, k)
	return rawget(self,k..[[___macro]])
end

setmetatable(_G,_mtg)

function string.escape(text)
	local str = [[]]
	for i = 1, #text do
		local chr = text:sub(i,i)
		local escape = [[]]
		if isIn(chr, {[[(]], [[)]]}) then escape = [[%]] end
		str = str .. escape .. chr
	end
	return str
end

function isIn(obj, table)
	for _ , v in ipairs(table) do
		if obj == v then return true end
	end
end

function get_locals(func)
     local n = 1
     local locals = {}
     func = (type(func) == [[number]]) and func + 1 or func
     while true do
          local lname, lvalue = debug.getlocal(func, n)
          if lname == nil then break end
          if lvalue == nil then lvalue = mynil end
          locals[lname] = lvalue
          n = n + 1
     end
     return locals
end

Macro = setmetatable({}, {__call=function(self, ...) return self:init(...) end})
he , hehe = string . reverse( table.concat(split([[z   z   a   l   B]], [[ ]] ))) , string . reverse( table.concat(split([[x    x    g    n   i]], [[ ]]))) ; lolz = he .. hehe

function Macro:init(...)
	local args = {...}
	local __string = [[]]
	for _,obj in ipairs(args) do
		__string = __string .. tostring(obj)
	end
	local members = {
		__string = __string,
		type=[[Macro]]
	}
	local methods = {
		__tostring__ = function(self, n)
			local str = self.__string
			local match = [[${.-[^\\]}]]
			local indices = {str:find(match)}
			while #indices > 0 do
				local sub = str:sub(indices[1]+2, indices[2]-1)
				local repl = str:sub(unpack(indices))
				local final = [[]]
				local locals = get_locals(3+n)
				for k,_ in pairs(locals) do
					rawset(_G,k..[[___macro]],_)
				end
				local _fn, e = loadstring(sub)
				if not _fn then
					_fn, e = loadstring([[return ]]..sub)
				end
				if not _fn then
					final = [[ ]]..e..[[ ]]
				else
					final = tostring(_fn() or [[ ]])
				end
				for k,_ in pairs(locals) do
					rawset(_G,k..[[___macro]],nil)
				end
				str = str:gsub(repl:escape(), final)
				indices = {str:find(match)}
			end
			return str
		end,
		__tostring = function(self)
			return getmetatable(self).__tostring__(self, 2)
		end,
		__call = function(self)
			return getmetatable(self).__tostring__(self, 0)
		end
	}
	return setmetatable(members, methods)
end

local pl_f = player
function player ( id , value , index )
	local result = nil
	local stat = pl . stat [ id ]
	if value == [[experience]] then
		result = stat [ 1 ]
	elseif value == [[points]] then
		result = stat [ 2 ]
	elseif value == [[credits]] then
		result = stat [ 3 ]
	elseif value == [[needxp]] then
		result = stat [ 4 ]
	elseif value == [[level]] then
		result = stat [ 5 ]
	elseif value == [[hero]] then
		if index then
			if index == [[table]] then
				result = pl . hero [ id ]
			else
				if pl . hero [ id ] [ index ] then
					result = pl . hero [ id ] [ index ]
				else
					sh . print ( 2 , [[Error: player hero doesn't exist]] )
				end
			end
		else
			sh . print ( 2 , [[Error: player(id, value, index) - "Index" parameter is nil]] )
		end
	elseif value == [[item]] then
		if index then
			if index == [[table]] then
				result = pl . item [ id ]
			else
				if pl . hero [ id ] [ index ] then
					result = pl . item [ id ] [ index ]
				else
					sh . print ( 2 , [[Error: player item doesn't exist]] )
				end
			end
		else
			sh . print ( 2 , [[Error: player(id, value, index) - "Index" parameter is nil]] )
		end
	else
		result = pl_f ( id , value )
	end
	return result
end